@@ -105,7 +105,7 @@ class lar_solver : public column_namer {
     indexed_vector<mpq> m_column_buffer;
     std::unordered_map<lar_term, std::pair<mpq, unsigned>, term_hasher, term_comparer>
         m_normalized_terms_to_columns;
-    vector<impq> m_backup_x;
+
     stacked_vector<unsigned> m_usage_in_terms;
     // ((x[j], is_int(j))->j)  for fixed j, used in equalities propagation
     // maps values to integral fixed vars
@@ -139,6 +139,8 @@ class lar_solver : public column_namer {
     bool compare_values(impq const& lhs, lconstraint_kind k, const mpq& rhs);
 
     inline void clear_columns_with_changed_bounds() { m_columns_with_changed_bounds.reset(); }
+
+    struct scoped_backup;
  public:
     const auto& columns_with_changed_bounds() const { return m_columns_with_changed_bounds; }
     void insert_to_columns_with_changed_bounds(unsigned j);
@@ -184,9 +186,7 @@ class lar_solver : public column_namer {
 
         return bound_analyzer_on_row<row_strip<mpq>, lp_bound_propagator<T>>::analyze_row(
             A_r().m_rows[row_index],
-            null_ci,
             zero_of_type<numeric_pair<mpq>>(),
-            row_index,
             bp);
     }
 
@@ -307,10 +307,9 @@ public:
 
     void get_infeasibility_explanation(explanation&) const;
 
-    inline void backup_x() { m_backup_x = m_mpq_lar_core_solver.m_r_x; }
 
-    inline void restore_x() { m_mpq_lar_core_solver.m_r_x = m_backup_x; }
 
+    std::function<void(lpvar)> m_fixed_var_eh;
     template <typename T>
     void explain_implied_bound(const implied_bound& ib, lp_bound_propagator<T>& bp) {
         u_dependency* dep = ib.explain_implied();
@@ -351,7 +350,15 @@ public:
      * return the rest of the row as t comprising of non-fixed variables and coeff as sum of fixed variables.
      * return false if j has no rows.
      */
-    bool solve_for(unsigned j, lar_term& t, mpq& coeff);
+
+    struct solution {
+        unsigned j;
+        lar_term t;
+    };
+
+    void solve_for(unsigned_vector const& js, vector<solution>& sol);
+    void check_fixed(unsigned j);
+    void solve_for(unsigned j, uint_set& tabu, vector<solution>& sol);
 
     inline unsigned get_base_column_in_row(unsigned row_index) const {
         return m_mpq_lar_core_solver.m_r_solver.get_base_column_in_row(row_index);
@@ -396,6 +403,10 @@ public:
                 }
     }
 
+    
+    public:
+    std::function<void (const lar_term*)> m_add_term_callback;
+    std::function<void (unsigned)> m_update_column_bound_callback;  
     bool external_is_used(unsigned) const;
     void pop(unsigned k);
     unsigned num_scopes() const { return m_trail.get_num_scopes(); }
@@ -443,7 +454,7 @@ public:
                                             const impq& new_val,
                                             const ChangeReport& after) {
         lp_assert(!is_base(j));
-        auto& x = m_mpq_lar_core_solver.m_r_x[j];
+        auto& x = m_mpq_lar_core_solver.r_x(j);
         auto delta = new_val - x;
         x = new_val;
         after(j);
@@ -501,7 +512,7 @@ public:
     u_dependency_manager& dep_manager() { return m_dependencies; }
 
     inline u_dependency* get_column_upper_bound_witness(unsigned j) const {
-         return m_columns[j].upper_bound_witness();
+        return m_columns[j].upper_bound_witness();
     }
 
     inline const impq& get_upper_bound(lpvar j) const {
@@ -518,6 +529,8 @@ public:
     
     bool has_lower_bound(lpvar var, u_dependency*& ci, mpq& value, bool& is_strict) const;
     bool has_upper_bound(lpvar var, u_dependency*& ci, mpq& value, bool& is_strict) const;
+    bool has_bound_of_type(lpvar var, u_dependency*& ci, mpq& value, bool& is_strict, bool is_upper) const;
+  
     bool has_value(lpvar var, mpq& value) const;
     bool fetch_normalized_term_column(const lar_term& t, std::pair<mpq, lpvar>&) const;
     bool column_is_fixed(unsigned j) const;
@@ -527,6 +540,9 @@ public:
     lp_settings const& settings() const;
     statistics& stats();
 
+    void backup_x() { m_mpq_lar_core_solver.backup_x(); }
+    void restore_x() { m_mpq_lar_core_solver.restore_x(); }
+
     void updt_params(params_ref const& p);
     column_type get_column_type(unsigned j) const { return m_mpq_lar_core_solver.m_column_types()[j]; }
     const vector<column_type>&  get_column_types() const { return m_mpq_lar_core_solver.m_column_types(); }
@@ -582,6 +598,15 @@ public:
         }
         return dep;
     }
+
+    std::ostream& print_expl(std::ostream& out, const explanation& exp) const {
+        for (auto p : exp)
+            constraints().display(
+                out, [this](lpvar j) { return get_variable_name(j); }, p.ci());
+        return out;
+    }
+
+    void explain_fixed_column(unsigned j, explanation& ex);
     u_dependency* join_deps(u_dependency* a, u_dependency *b) { return m_dependencies.mk_join(a, b); }
     inline constraint_set const& constraints() const { return m_constraints; }
     void push();
@@ -591,21 +616,32 @@ public:
         return m_columns[j].lower_bound_witness();
     }
     inline bool column_has_term(lpvar j) const { return m_columns[j].term() != nullptr; }
-    inline std::ostream& print_column_info(unsigned j, std::ostream& out) const {
-        m_mpq_lar_core_solver.m_r_solver.print_column_info(j, out);
-        if (column_has_term(j)) {
-            print_term_as_indices(get_term(j), out) << "\n";
 
-        } else if (column_has_term(j)) {
-            const lar_term& t = get_term(m_var_register.local_to_external(j));
-            print_term_as_indices(t, out) << "\n";
-        }
+    std::ostream& print_column_info(unsigned j, std::ostream& out) const {
+        m_mpq_lar_core_solver.m_r_solver.print_column_info(j, out);
+        if (column_has_term(j)) 
+            print_term_as_indices(get_term(j), out) << "\n";       
+        display_column_explanation(out, j);
+        return out;
+    }
+
+    std::ostream& display_column_explanation(std::ostream& out, unsigned j) const {
+        const column& ul = m_columns[j];
+        svector<unsigned> vs1, vs2;
+        m_dependencies.linearize(ul.lower_bound_witness(), vs1);
+        m_dependencies.linearize(ul.upper_bound_witness(), vs2);
+        if (!vs1.empty())
+            out << "lo: " << vs1;
+        if (!vs2.empty())
+            out << "hi: " << vs2;
+        if (!vs1.empty() || !vs2.empty())
+            out << "\n";
         return out;
     }
 
     void subst_known_terms(lar_term*);
 
-    inline std::ostream& print_column_bound_info(unsigned j, std::ostream& out) const {
+    std::ostream& print_column_bound_info(unsigned j, std::ostream& out) const {
         return m_mpq_lar_core_solver.m_r_solver.print_column_bound_info(j, out);
     }
 
@@ -624,6 +660,7 @@ public:
     inline int_solver* get_int_solver() { return m_int_solver; }
     inline const int_solver* get_int_solver() const { return m_int_solver; }
     inline const lar_term& get_term(lpvar j) const {
+        SASSERT(column_has_term(j));
         return *m_columns[j].term();
     }
     lp_status find_feasible_solution();
@@ -645,7 +682,7 @@ public:
     bool ax_is_correct() const;
     bool get_equality_and_right_side_for_term_on_current_x(lpvar j, mpq& rs, u_dependency*& ci, bool& upper_bound) const;
     bool var_is_int(lpvar v) const;
-    inline const vector<int>& r_heading() const { return m_mpq_lar_core_solver.m_r_heading; }
+    inline const std_vector<int>& r_heading() const { return m_mpq_lar_core_solver.m_r_heading; }
     inline const vector<unsigned>& r_basis() const { return m_mpq_lar_core_solver.r_basis(); }
     inline const vector<unsigned>& r_nbasis() const { return m_mpq_lar_core_solver.r_nbasis(); }
     inline bool column_is_real(unsigned j) const { return !column_is_int(j); }
@@ -654,19 +691,18 @@ public:
     void set_status(lp_status s);
     lp_status solve();
     void fill_explanation_from_crossed_bounds_column(explanation& evidence) const;
-    bool term_is_used_as_row(unsigned term) const;
     bool tighten_term_bounds_by_delta(lpvar j, const impq&);
     lar_solver();
     void track_touched_rows(bool v);
     bool touched_rows_are_tracked() const;
     ~lar_solver() override;
-    const vector<impq>& r_x() const { return m_mpq_lar_core_solver.m_r_x; }
+    const vector<impq>& r_x() const { return m_mpq_lar_core_solver.r_x(); }
     bool column_is_int(unsigned j) const;
-    inline bool column_value_is_int(unsigned j) const { return m_mpq_lar_core_solver.m_r_x[j].is_int(); }
+    inline bool column_value_is_int(unsigned j) const { return m_mpq_lar_core_solver.r_x(j).is_int(); }
     inline static_matrix<mpq, impq>& A_r() { return m_mpq_lar_core_solver.m_r_A; }
     inline const static_matrix<mpq, impq>& A_r() const { return m_mpq_lar_core_solver.m_r_A; }
     // columns
-    const impq& get_column_value(lpvar j) const { return m_mpq_lar_core_solver.m_r_x[j]; }
+    const impq& get_column_value(lpvar j) const { return m_mpq_lar_core_solver.r_x(j); }
     inline lpvar external_to_local(unsigned j) const {
         lpvar local_j;
         if (m_var_register.external_is_used(j, local_j)) {
