@@ -18,6 +18,8 @@ Revision History:
 
 --*/
 #pragma once
+#include <string>
+#include "util/debug.h"
 namespace lp {
     enum class lia_move {
         sat,
@@ -26,7 +28,8 @@ namespace lp {
         conflict,
         continue_with_check,
         undef,
-        unsat
+        unsat,
+        cancelled
     };
     inline std::string lia_move_to_string(lia_move m) {
         switch (m) {
