@@ -2456,7 +2456,7 @@ namespace qe {
     }
 
     bool has_quantified_uninterpreted(ast_manager& m, expr* fml) {
-        struct found {};
+        struct found : public std::exception {};
         struct proc {
             ast_manager& m;
             proc(ast_manager& m):m(m) {}
@@ -2472,7 +2472,7 @@ namespace qe {
             for_each_expr(p, fml);
             return false;
         }
-        catch (found) {
+        catch (found const&) {
             return true;
         }
     }
