@@ -1176,7 +1176,7 @@ namespace arith {
             TRACE("arith", tout << "branch\n";);
             app_ref b(m);
             bool u = m_lia->is_upper();
-            auto const& k = m_lia->get_offset();
+            auto const& k = m_lia->offset();
             rational offset;
             expr_ref t(m);
             b = mk_bound(m_lia->get_term(), k, !u, offset, t);
@@ -1192,14 +1192,14 @@ namespace arith {
         }
         case lp::lia_move::cut: {
             TRACE("arith", tout << "cut\n";);
-            ++m_stats.m_gomory_cuts;
+            ++m_stats.m_cuts;
             // m_explanation implies term <= k
             reset_evidence();
             for (auto ev : m_explanation)
                 set_evidence(ev.ci());
             // The call mk_bound() can set the m_infeasible_column in lar_solver
             // so the explanation is safer to take before this call.
-            app_ref b = mk_bound(m_lia->get_term(), m_lia->get_offset(), !m_lia->is_upper());
+            app_ref b = mk_bound(m_lia->get_term(), m_lia->offset(), !m_lia->is_upper());
             IF_VERBOSE(4, verbose_stream() << "cut " << b << "\n");
             literal lit = expr2literal(b);
             assign(lit, m_core, m_eqs, explain(hint_type::cut_h, lit));
