@@ -20,6 +20,7 @@ Revision History:
 #include "smt/smt_context.h"
 #include "smt/smt_lookahead.h"
 #include "ast/ast_smt2_pp.h"
+#include "ast/ast_util.h"
 #include "smt/params/smt_params_helper.hpp"
 
 namespace smt {
@@ -213,11 +214,14 @@ namespace smt {
         return out;
     }
 
-    bool kernel::solve_for(expr* e, expr_ref& term) {
-        smt::enode* n = m_imp->m_kernel.find_enode(e);        
-        if (!n)
-            return false;
-        return m_imp->m_kernel.solve_for(n, term);        
+    void kernel::solve_for(vector<solver::solution>& sol) {
+        vector<smt::solution> solution;
+        for (auto const& [v, t, g] : sol)
+            solution.push_back({ v, t, g });
+        m_imp->m_kernel.solve_for(solution);
+        sol.reset();
+        for (auto s : solution)
+            sol.push_back({ s.var, s.term, s.guard });       
     }
 
     expr* kernel::congruence_root(expr * e) {
@@ -234,6 +238,21 @@ namespace smt {
         return n->get_next()->get_expr();
     }
 
+    expr_ref kernel::congruence_explain(expr* a, expr* b) {
+        auto& ctx = m_imp->m_kernel;
+        ast_manager& m = ctx.get_manager();
+        smt::enode* n1 = ctx.find_enode(a);
+        smt::enode* n2 = ctx.find_enode(b);
+        if (!n1 || !n2 || n1->get_root() != n2->get_root())
+            return expr_ref(m.mk_eq(a, b), m);
+        literal_vector lits;
+        ctx.get_cr().eq2literals(n1, n2, lits);
+        expr_ref_vector es(m);
+        for (auto lit : lits)
+            es.push_back(ctx.literal2expr(lit));        
+        return mk_and(es);
+    }
+
     void kernel::collect_statistics(::statistics & st) const {
         m_imp->m_kernel.collect_statistics(st);
     }
